defprotocol Bamboo.Formatter do
  @moduledoc ~S"""
  Converts data to a Bamboo.EmailAddress struct or a string

  The passed in options is currently a map with the key `:type` and a value of
  `:from`, `:to`, `:cc` or `:bcc`. This makes it so that you can pattern match
  and return a different address depending on if the address is being used in
  the from, to, cc or bcc.

  ## Simple example

  Let's say you have a user struct like this.

      defmodule MyApp.User do
        defstruct first_name: nil, last_name: nil, email: nil
      end

  Bamboo can automatically format this struct if you implement the Bamboo.Formatter
  protocol.

      defimpl Bamboo.Formatter, for: MyApp.User do
        # Used by `to`, `bcc`, `cc` and `from`
        def format_email_address(user, _opts) do
          fullname = "#{user.first_name} #{user.last_name}"
          %Bamboo.EmailAddress{name: fullname, email: email}
        end
      end

  Now you can create emails like this, and the user will be formatted correctly

      user = %User{first_name: "John", last_name: "Doe", email: "me@example.com"}
      Bamboo.Email.new_email(from: user)
  """

  @doc ~S"""
  Receives data and opts and should return a string or Bamboo.EmailAddress struct

  opts is currently a map with the key `:type` and a value of
  `:from`, `:to`, `:cc` or `:bcc`. You can pattern match on this to customize
  the address depending on what it's use for. For example, you could add the
  app name to the user's name if it is used for a from address

      defimpl Bamboo.Formatter, for: MyApp.User do
        # Include the app name when used in a from address
        def format_email_address(user, %{type: :from}) do
          fullname = "#{user.first_name} #{user.last_name} (Send from MyApp)"
          %Bamboo.EmailAddress{name: fullname, email: email}
        end

        # Just use the name for all other types
        def format_email_address(user, _opts) do
          fullname = "#{user.first_name} #{user.last_name}"
          %Bamboo.EmailAddress{name: fullname, email: email}
        end
      end
  """
  def format_email_address(data, opts)
end

defimpl Bamboo.Formatter, for: List do
  def format_email_address(email_addresses, opts) do
    email_addresses |> Enum.map(&Bamboo.Formatter.format_email_address(&1, opts))
  end
end

defimpl Bamboo.Formatter, for: BitString do
  def format_email_address(email_address, _opts) do
    %Bamboo.EmailAddress{name: nil, address: email_address}
  end
end

defimpl Bamboo.Formatter, for: Bamboo.EmailAddress do
  def format_email_address(already_formatted_email, _opts) do
    already_formatted_email
  end
end

defimpl Bamboo.Formatter, for: Map do
  def format_email_address(invalid_address, _opts) do
    raise ArgumentError, """
    The format of the address was invalid. Got #{inspect invalid_address}.
    Expected a string, e.g. "foo@bar.com", or a Bamboo.EmailAddress struct.

    You can also implement a custom protocol for structs.

    Example:

    defimpl Bamboo.Formatter, for: MyApp.User do
      def format_email_address(user, _opts) do
        %Bamboo.EmailAddress{name: user.name, address: user.email}
      end
    end
    """
  end
end
